#ifndef __DBUS_ADAPTER_CONEXUS_DBUS_IPV4_TCPSERVER_H
#define __DBUS_ADAPTER_CONEXUS_DBUS_IPV4_TCPSERVER_H

#include <dbus-cxx.h>

#include <conexus/ipv4_tcpserver.h>

#include <conexus-dbus/server_adapter.h>

#include <conexus-dbus/ipv4_address_adapter.h>
#include <conexus-dbus/ipv4_tcp_adapter.h>

namespace Conexus {
  namespace DBus {
    namespace IPv4 {

      class TCPServerAdapter : public ServerAdapter
      {
        protected:
          TCPServerAdapter( Conexus::IPv4::TCPServer* adaptee=NULL, const std::string& path="/Conexus/IPv4/TCPServer"):
            ServerAdapter(adaptee, path),
            m_adaptee(adaptee)
          {

            ::DBus::MethodBase::pointer temp_method;

            temp_method = this->create_method<void,uint32_t,uint16_t>( "Conexus.IPv4.TCPServer", "set_address", sigc::mem_fun(*this, &TCPServerAdapter::set_address_adapter_stub_vuq) );
            temp_method->set_arg_name( 1, "addr" );
            temp_method->set_arg_name( 2, "port" );
            temp_method = this->create_method<void,std::string,uint16_t>( "Conexus.IPv4.TCPServer", "set_address", sigc::mem_fun(*this, &TCPServerAdapter::set_address_adapter_stub_vsq) );
            temp_method->set_arg_name( 1, "addr" );
            temp_method->set_arg_name( 2, "port" );
            temp_method = this->create_method<int32_t>( "Conexus.IPv4.TCPServer", "backlog", sigc::mem_fun(*this, &TCPServerAdapter::backlog_adapter_stub_i) );
            temp_method->set_arg_name( 0, "value" );
            temp_method = this->create_method<void,bool>( "Conexus.IPv4.TCPServer", "enable_reuse_address", sigc::mem_fun(*this, &TCPServerAdapter::enable_reuse_address_adapter_stub_vb) );
            temp_method->set_arg_name( 1, "enable" );

            ::DBus::Path child_path;
            
            child_path = path;
            child_path.append_element("Address");
            DBUS_CXX_DEBUG( "Address:constructor: child path = " << child_path);
            m_Address = AddressAdapter::create(NULL, child_path);            this->add_child("Address", m_Address);
            if ( adaptee ) m_Address->set_adaptee(&(adaptee->address()));
            
            child_path = path;
            child_path.append_element("Listener");
            DBUS_CXX_DEBUG( "Listener:constructor: child path = " << child_path);
            m_Listener = TCPAdapter::create(NULL, child_path);            this->add_child("Listener", m_Listener);
            if ( adaptee ) m_Listener->set_adaptee(&(adaptee->listener_ref()));
          }

        public:

          typedef DBusCxxPointer<TCPServerAdapter> pointer;

          static pointer create( Conexus::IPv4::TCPServer* adaptee=NULL, const std::string& path="/Conexus/IPv4/TCPServer")
            { return pointer( new TCPServerAdapter(adaptee, path)); }

          Conexus::IPv4::TCPServer* adaptee() { return m_adaptee; }

          void set_adaptee( Conexus::IPv4::TCPServer* adaptee ) {
            m_adaptee = adaptee;
            if ( m_adaptee ) {
              m_Address->set_adaptee( m_adaptee->address() );
              m_Listener->set_adaptee( m_adaptee->listener_ref() );
            }
            else {
              m_Address->set_adaptee( NULL );
              m_Listener->set_adaptee( NULL );
            }
          }

          AddressAdapter::pointer address() { return m_Address; }

          TCPAdapter::pointer listener() { return m_Listener; }

          void set_adaptee( Conexus::IPv4::TCPServer& adaptee ) {
            this->set_adaptee(&adaptee);
          }


        private:

          Conexus::IPv4::TCPServer* m_adaptee;

          AddressAdapter::pointer m_Address;
          TCPAdapter::pointer m_Listener;

          void check_adaptee() { if ( not m_adaptee) throw ::DBus::ErrorInvalidAdaptee::create(); }

          void set_address_adapter_stub_vuq( uint32_t addr, uint16_t port ) { this->check_adaptee(); m_adaptee->set_address( addr, port); }
          void set_address_adapter_stub_vsq( std::string addr, uint16_t port ) { this->check_adaptee(); m_adaptee->set_address( addr, port); }
          int32_t backlog_adapter_stub_i( ) { this->check_adaptee(); return m_adaptee->backlog(); }
          void enable_reuse_address_adapter_stub_vb( bool enable ) { this->check_adaptee(); m_adaptee->enable_reuse_address( enable); }


      };

    }
  }
}

#endif
