#ifndef __DBUS_PROXY_CONEXUS_DBUS_IPV4_TCPSERVER_H
#define __DBUS_PROXY_CONEXUS_DBUS_IPV4_TCPSERVER_H

#include <dbus-cxx.h>

#include <conexus/ipv4_tcpserver.h>

#include <conexus-dbus/server_proxy.h>

#include <conexus-dbus/ipv4_address_proxy.h>
#include <conexus-dbus/ipv4_tcp_proxy.h>

namespace Conexus {
  namespace DBus {
    namespace IPv4 {

      class TCPServerProxy : public ServerProxy
      {
        protected:
          TCPServerProxy( ::DBus::Connection::pointer conn, const std::string& dest="Conexus.IPv4.TCPServer", const std::string& path="/Conexus/IPv4/TCPServer"):
            ServerProxy(conn, dest, path)
          {
            ::DBus::Path child_path;
            child_path = path; 
            child_path.append_element("Address");
            m_Address = AddressProxy::create(conn, dest, child_path);
            child_path = path; 
            child_path.append_element("Listener");
            m_Listener = TCPProxy::create(conn, dest, child_path);

            m_method_set_address_vuq = this->create_method< void,uint32_t,uint16_t >( "Conexus.IPv4.TCPServer", "set_address" );
            m_method_set_address_vsq = this->create_method< void,std::string,uint16_t >( "Conexus.IPv4.TCPServer", "set_address" );
            m_method_backlog_i = this->create_method< int32_t >( "Conexus.IPv4.TCPServer", "backlog" );
            m_method_enable_reuse_address_vb = this->create_method< void,bool >( "Conexus.IPv4.TCPServer", "enable_reuse_address" );
          }

        public:

          typedef DBusCxxPointer<TCPServerProxy> pointer;

          static pointer create( ::DBus::Connection::pointer conn,
                                 const std::string& dest="Conexus.IPv4.TCPServer",
                                 const std::string& path="/Conexus/IPv4/TCPServer"
                               )
          { return pointer( new TCPServerProxy(conn, dest, path)); }

          AddressProxy::pointer address() { return m_Address; }
          TCPProxy::pointer listener() { return m_Listener; }

          void set_address( uint32_t addr, uint16_t port ) { return (*m_method_set_address_vuq)( addr, port); }

          void set_address( std::string addr, uint16_t port ) { return (*m_method_set_address_vsq)( addr, port); }

          int32_t backlog( ) { return (*m_method_backlog_i)(); }

          void enable_reuse_address( bool enable ) { return (*m_method_enable_reuse_address_vb)( enable); }


        protected:

          AddressProxy::pointer m_Address;
          TCPProxy::pointer m_Listener;

          ::DBus::MethodProxy<void,uint32_t,uint16_t>::pointer m_method_set_address_vuq;
          ::DBus::MethodProxy<void,std::string,uint16_t>::pointer m_method_set_address_vsq;
          ::DBus::MethodProxy<int32_t>::pointer m_method_backlog_i;
          ::DBus::MethodProxy<void,bool>::pointer m_method_enable_reuse_address_vb;


      };

    }
  }
}

#endif
