#ifndef __DBUS_ADAPTER_CONEXUS_DBUS_IPV4_UDP_H
#define __DBUS_ADAPTER_CONEXUS_DBUS_IPV4_UDP_H

#include <dbus-cxx.h>

#include <conexus/ipv4_udp.h>

#include <conexus-dbus/ipv4_ip_adapter.h>


namespace Conexus {
  namespace DBus {
    namespace IPv4 {

      class UDPAdapter : public IPAdapter
      {
        protected:
          UDPAdapter( Conexus::IPv4::UDP* adaptee=NULL, const std::string& path="/Conexus/IPv4/UDP"):
            IPAdapter(adaptee, path),
            m_adaptee(adaptee)
          {

            ::DBus::MethodBase::pointer temp_method;

            temp_method = this->create_method<void,bool>( "Conexus.IPv4.UDP", "set_write_without_connect", sigc::mem_fun(*this, &UDPAdapter::set_write_without_connect_adapter_stub_vb) );
            temp_method->set_arg_name( 1, "value" );
            temp_method = this->create_method<bool>( "Conexus.IPv4.UDP", "write_without_connect", sigc::mem_fun(*this, &UDPAdapter::write_without_connect_adapter_stub_b) );
            temp_method->set_arg_name( 0, "value" );
            temp_method = this->create_method<void,uint32_t>( "Conexus.IPv4.UDP", "add_multicast_interface", sigc::mem_fun(*this, &UDPAdapter::add_multicast_interface_adapter_stub_vu) );
            temp_method->set_arg_name( 1, "iface_index" );
            temp_method = this->create_method<void,std::string>( "Conexus.IPv4.UDP", "add_multicast_interface", sigc::mem_fun(*this, &UDPAdapter::add_multicast_interface_adapter_stub_vs) );
            temp_method->set_arg_name( 1, "iface_index" );
            temp_method = this->create_method<void,uint32_t>( "Conexus.IPv4.UDP", "remove_multicast_interface", sigc::mem_fun(*this, &UDPAdapter::remove_multicast_interface_adapter_stub_vu) );
            temp_method->set_arg_name( 1, "iface_index" );
            temp_method = this->create_method<void,std::string>( "Conexus.IPv4.UDP", "remove_multicast_interface", sigc::mem_fun(*this, &UDPAdapter::remove_multicast_interface_adapter_stub_vs) );
            temp_method->set_arg_name( 1, "iface_index" );
            temp_method = this->create_method<bool,uint32_t>( "Conexus.IPv4.UDP", "is_multicast_interface", sigc::mem_fun(*this, &UDPAdapter::is_multicast_interface_adapter_stub_bu) );
            temp_method->set_arg_name( 0, "answer" );
            temp_method->set_arg_name( 1, "iface_index" );
            temp_method = this->create_method<bool,std::string>( "Conexus.IPv4.UDP", "is_multicast_interface", sigc::mem_fun(*this, &UDPAdapter::is_multicast_interface_adapter_stub_bs) );
            temp_method->set_arg_name( 0, "answer" );
            temp_method->set_arg_name( 1, "iface_index" );
            temp_method = this->create_method<void,bool>( "Conexus.IPv4.UDP", "set_suppress_multicast_join", sigc::mem_fun(*this, &UDPAdapter::set_suppress_multicast_join_adapter_stub_vb) );
            temp_method->set_arg_name( 1, "value" );
            temp_method = this->create_method<bool>( "Conexus.IPv4.UDP", "suppress_multicast_join", sigc::mem_fun(*this, &UDPAdapter::suppress_multicast_join_adapter_stub_b) );
            temp_method->set_arg_name( 0, "value" );
            temp_method = this->create_method<void,uint32_t>( "Conexus.IPv4.UDP", "set_multicast_hop_limit", sigc::mem_fun(*this, &UDPAdapter::set_multicast_hop_limit_adapter_stub_vu) );
            temp_method->set_arg_name( 1, "limit" );
            temp_method = this->create_method<uint32_t>( "Conexus.IPv4.UDP", "multicast_hop_limit", sigc::mem_fun(*this, &UDPAdapter::multicast_hop_limit_adapter_stub_u) );
            temp_method->set_arg_name( 0, "limit" );
            temp_method = this->create_method<void,uint32_t>( "Conexus.IPv4.UDP", "set_default_multicast_hop_limit", sigc::mem_fun(*this, &UDPAdapter::set_default_multicast_hop_limit_adapter_stub_vu) );
            temp_method->set_arg_name( 1, "limit" );
            temp_method = this->create_method<uint32_t>( "Conexus.IPv4.UDP", "default_multicast_hop_limit", sigc::mem_fun(*this, &UDPAdapter::default_multicast_hop_limit_adapter_stub_u) );
            temp_method->set_arg_name( 0, "limit" );
            temp_method = this->create_method<bool>( "Conexus.IPv4.UDP", "is_multicast_loopback_enabled", sigc::mem_fun(*this, &UDPAdapter::is_multicast_loopback_enabled_adapter_stub_b) );
            temp_method->set_arg_name( 0, "enabled" );
            temp_method = this->create_method<void,bool>( "Conexus.IPv4.UDP", "enable_multicast_loopback", sigc::mem_fun(*this, &UDPAdapter::enable_multicast_loopback_adapter_stub_vb) );
            temp_method->set_arg_name( 1, "enabled" );
            temp_method = this->create_method<bool>( "Conexus.IPv4.UDP", "is_default_multicast_loopback_enabled", sigc::mem_fun(*this, &UDPAdapter::is_default_multicast_loopback_enabled_adapter_stub_b) );
            temp_method->set_arg_name( 0, "enabled" );
            temp_method = this->create_method<void,bool>( "Conexus.IPv4.UDP", "enable_default_multicast_loopback", sigc::mem_fun(*this, &UDPAdapter::enable_default_multicast_loopback_adapter_stub_vb) );
            temp_method->set_arg_name( 1, "enabled" );
          }

        public:

          typedef DBusCxxPointer<UDPAdapter> pointer;

          static pointer create( Conexus::IPv4::UDP* adaptee=NULL, const std::string& path="/Conexus/IPv4/UDP")
            { return pointer( new UDPAdapter(adaptee, path)); }

          Conexus::IPv4::UDP* adaptee() { return m_adaptee; }

          void set_adaptee( Conexus::IPv4::UDP* adaptee ) {
            m_adaptee = adaptee;
          }

          void set_adaptee( Conexus::IPv4::UDP& adaptee ) {
            this->set_adaptee(&adaptee);
          }


        private:

          Conexus::IPv4::UDP* m_adaptee;


          void check_adaptee() { if ( not m_adaptee) throw ::DBus::ErrorInvalidAdaptee::create(); }

          void set_write_without_connect_adapter_stub_vb( bool value ) { this->check_adaptee(); m_adaptee->set_write_without_connect( value); }
          bool write_without_connect_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->write_without_connect(); }
          void add_multicast_interface_adapter_stub_vu( uint32_t iface_index ) { this->check_adaptee(); m_adaptee->add_multicast_interface( iface_index); }
          void add_multicast_interface_adapter_stub_vs( std::string iface_index ) { this->check_adaptee(); m_adaptee->add_multicast_interface( iface_index); }
          void remove_multicast_interface_adapter_stub_vu( uint32_t iface_index ) { this->check_adaptee(); m_adaptee->remove_multicast_interface( iface_index); }
          void remove_multicast_interface_adapter_stub_vs( std::string iface_index ) { this->check_adaptee(); m_adaptee->remove_multicast_interface( iface_index); }
          bool is_multicast_interface_adapter_stub_bu( uint32_t iface_index ) { this->check_adaptee(); return m_adaptee->is_multicast_interface( iface_index); }
          bool is_multicast_interface_adapter_stub_bs( std::string iface_index ) { this->check_adaptee(); return m_adaptee->is_multicast_interface( iface_index); }
          void set_suppress_multicast_join_adapter_stub_vb( bool value ) { this->check_adaptee(); m_adaptee->set_suppress_multicast_join( value); }
          bool suppress_multicast_join_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->suppress_multicast_join(); }
          void set_multicast_hop_limit_adapter_stub_vu( uint32_t limit ) { this->check_adaptee(); m_adaptee->set_multicast_hop_limit( limit); }
          uint32_t multicast_hop_limit_adapter_stub_u( ) { this->check_adaptee(); return m_adaptee->multicast_hop_limit(); }
          void set_default_multicast_hop_limit_adapter_stub_vu( uint32_t limit ) { this->check_adaptee(); m_adaptee->set_default_multicast_hop_limit( limit); }
          uint32_t default_multicast_hop_limit_adapter_stub_u( ) { this->check_adaptee(); return m_adaptee->default_multicast_hop_limit(); }
          bool is_multicast_loopback_enabled_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->is_multicast_loopback_enabled(); }
          void enable_multicast_loopback_adapter_stub_vb( bool enabled ) { this->check_adaptee(); m_adaptee->enable_multicast_loopback( enabled); }
          bool is_default_multicast_loopback_enabled_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->is_default_multicast_loopback_enabled(); }
          void enable_default_multicast_loopback_adapter_stub_vb( bool enabled ) { this->check_adaptee(); m_adaptee->enable_default_multicast_loopback( enabled); }


      };

    }
  }
}

#endif
