#ifndef __DBUS_ADAPTER_CONEXUS_DBUS_SERIAL_H
#define __DBUS_ADAPTER_CONEXUS_DBUS_SERIAL_H

#include <dbus-cxx.h>

#include <conexus/serial.h>

#include <conexus-dbus/filedescriptor_adapter.h>

#ifndef __DBUS_CONEXUS__READWRITE_ITERATOR_SUPPORT
  #define __DBUS_CONEXUS__READWRITE_ITERATOR_SUPPORT
  DBUS_CXX_ITERATOR_SUPPORT( Conexus::ReadWrite , int32_t )
#endif


namespace Conexus {
  namespace DBus {

    class SerialAdapter : public FileDescriptorAdapter
    {
      protected:
        SerialAdapter( Conexus::Serial* adaptee=NULL, const std::string& path="/Conexus/Serial"):
          FileDescriptorAdapter(adaptee, path),
          m_adaptee(adaptee)
        {

          ::DBus::MethodBase::pointer temp_method;

          temp_method = this->create_method<void>( "Conexus.Serial", "open", sigc::mem_fun(*this, &SerialAdapter::open_adapter_stub_v) );
          temp_method = this->create_method<void,std::string>( "Conexus.Serial", "open", sigc::mem_fun(*this, &SerialAdapter::open_adapter_stub_vs) );
          temp_method->set_arg_name( 1, "device" );
          temp_method = this->create_method<void,std::string,Conexus::ReadWrite>( "Conexus.Serial", "open", sigc::mem_fun(*this, &SerialAdapter::open_adapter_stub_vsi) );
          temp_method->set_arg_name( 1, "device" );
          temp_method->set_arg_name( 2, "read_write_mode" );
          temp_method = this->create_method<uint32_t>( "Conexus.Serial", "rw_mode", sigc::mem_fun(*this, &SerialAdapter::rw_mode_adapter_stub_u) );
          temp_method->set_arg_name( 0, "m" );
          temp_method = this->create_method<std::string>( "Conexus.Serial", "device", sigc::mem_fun(*this, &SerialAdapter::device_adapter_stub_s) );
          temp_method->set_arg_name( 0, "d" );
          temp_method = this->create_method<void,std::string>( "Conexus.Serial", "set_device", sigc::mem_fun(*this, &SerialAdapter::set_device_adapter_stub_vs) );
          temp_method->set_arg_name( 1, "device" );
          temp_method = this->create_method<void,std::string,Conexus::ReadWrite>( "Conexus.Serial", "set_device", sigc::mem_fun(*this, &SerialAdapter::set_device_adapter_stub_vsi) );
          temp_method->set_arg_name( 1, "device" );
          temp_method->set_arg_name( 2, "read_write_mode" );
          m_signal_adapter_device_changed = this->create_signal<void>("Conexus.Serial","device_changed");
          if ( m_adaptee ) m_signal_adapter_connection_device_changed = m_adaptee->signal_device_changed().connect( *m_signal_adapter_device_changed );
        }

      public:

        typedef DBusCxxPointer<SerialAdapter> pointer;

        static pointer create( Conexus::Serial* adaptee=NULL, const std::string& path="/Conexus/Serial")
          { return pointer( new SerialAdapter(adaptee, path)); }

        Conexus::Serial* adaptee() { return m_adaptee; }

        void set_adaptee( Conexus::Serial* adaptee ) {
          m_adaptee = adaptee;
          m_signal_adapter_connection_device_changed.disconnect();
          if ( m_adaptee ) m_signal_adapter_connection_device_changed = m_adaptee->signal_device_changed().connect( *m_signal_adapter_device_changed );
        }

        void set_adaptee( Conexus::Serial& adaptee ) {
          this->set_adaptee(&adaptee);
        }


      private:

        Conexus::Serial* m_adaptee;


        void check_adaptee() { if ( not m_adaptee) throw ::DBus::ErrorInvalidAdaptee::create(); }

        void open_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->open(); }
        void open_adapter_stub_vs( std::string device ) { this->check_adaptee(); m_adaptee->open( device); }
        void open_adapter_stub_vsi( std::string device, Conexus::ReadWrite read_write_mode ) { this->check_adaptee(); m_adaptee->open( device, read_write_mode); }
        uint32_t rw_mode_adapter_stub_u( ) { this->check_adaptee(); return m_adaptee->rw_mode(); }
        std::string device_adapter_stub_s( ) { this->check_adaptee(); return m_adaptee->device(); }
        void set_device_adapter_stub_vs( std::string device ) { this->check_adaptee(); m_adaptee->set_device( device); }
        void set_device_adapter_stub_vsi( std::string device, Conexus::ReadWrite read_write_mode ) { this->check_adaptee(); m_adaptee->set_device( device, read_write_mode); }
        ::DBus::signal<void>::pointer m_signal_adapter_device_changed;
        sigc::connection m_signal_adapter_connection_device_changed;


    };

  }
}

#endif
