#ifndef __DBUS_ADAPTER_CONEXUS_DBUS_SERVER_H
#define __DBUS_ADAPTER_CONEXUS_DBUS_SERVER_H

#include <dbus-cxx.h>

#include <conexus/server.h>


namespace Conexus {
  namespace DBus {

    class ServerAdapter : public ::DBus::Object
    {
      protected:
        ServerAdapter( Conexus::Server* adaptee=NULL, const std::string& path="/Conexus/Server"):
          ::DBus::Object(path),
          m_adaptee(adaptee)
        {

          ::DBus::MethodBase::pointer temp_method;

          temp_method = this->create_method<void>( "Conexus.Server", "start", sigc::mem_fun(*this, &ServerAdapter::start_adapter_stub_v) );
          temp_method = this->create_method<void,bool>( "Conexus.Server", "start", sigc::mem_fun(*this, &ServerAdapter::start_adapter_stub_vb) );
          temp_method->set_arg_name( 1, "use_dispatcher" );
          temp_method = this->create_method<void>( "Conexus.Server", "stop", sigc::mem_fun(*this, &ServerAdapter::stop_adapter_stub_v) );
          temp_method = this->create_method<void,int64_t>( "Conexus.Server", "set_responsiveness", sigc::mem_fun(*this, &ServerAdapter::set_responsiveness_adapter_stub_vx) );
          temp_method->set_arg_name( 1, "response_us" );
          temp_method = this->create_method<int64_t>( "Conexus.Server", "responsiveness", sigc::mem_fun(*this, &ServerAdapter::responsiveness_adapter_stub_x) );
          temp_method->set_arg_name( 0, "response_us" );
          temp_method = this->create_method<bool>( "Conexus.Server", "is_running", sigc::mem_fun(*this, &ServerAdapter::is_running_adapter_stub_b) );
          temp_method->set_arg_name( 0, "result" );
          m_signal_adapter_start_stop = this->create_signal<void,int32_t>("Conexus.Server","start_stop");
          m_signal_adapter_start_stop->set_arg_name( 0, "condition" );
          if ( m_adaptee ) m_signal_adapter_connection_start_stop = m_adaptee->signal_start_stop().connect( *m_signal_adapter_start_stop );
        }

      public:

        typedef DBusCxxPointer<ServerAdapter> pointer;

        static pointer create( Conexus::Server* adaptee=NULL, const std::string& path="/Conexus/Server")
          { return pointer( new ServerAdapter(adaptee, path)); }

        Conexus::Server* adaptee() { return m_adaptee; }

        void set_adaptee( Conexus::Server* adaptee ) {
          m_adaptee = adaptee;
          m_signal_adapter_connection_start_stop.disconnect();
          if ( m_adaptee ) m_signal_adapter_connection_start_stop = m_adaptee->signal_start_stop().connect( *m_signal_adapter_start_stop );
        }

        void set_adaptee( Conexus::Server& adaptee ) {
          this->set_adaptee(&adaptee);
        }


      private:

        Conexus::Server* m_adaptee;


        void check_adaptee() { if ( not m_adaptee) throw ::DBus::ErrorInvalidAdaptee::create(); }

        void start_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->start(); }
        void start_adapter_stub_vb( bool use_dispatcher ) { this->check_adaptee(); m_adaptee->start( use_dispatcher); }
        void stop_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->stop(); }
        void set_responsiveness_adapter_stub_vx( int64_t response_us ) { this->check_adaptee(); m_adaptee->set_responsiveness( response_us); }
        int64_t responsiveness_adapter_stub_x( ) { this->check_adaptee(); return m_adaptee->responsiveness(); }
        bool is_running_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->is_running(); }
        ::DBus::signal<void,int32_t>::pointer m_signal_adapter_start_stop;
        sigc::connection m_signal_adapter_connection_start_stop;


    };

  }
}

#endif
