#ifndef __DBUS_ADAPTER_CONEXUS_DBUS_TTY_H
#define __DBUS_ADAPTER_CONEXUS_DBUS_TTY_H

#include <dbus-cxx.h>

#include <conexus/tty.h>

#include <conexus-dbus/serial_adapter.h>

#ifndef __DBUS_CONEXUS__FLOWCONTROL_ITERATOR_SUPPORT
  #define __DBUS_CONEXUS__FLOWCONTROL_ITERATOR_SUPPORT
  DBUS_CXX_ITERATOR_SUPPORT( Conexus::FlowControl , int32_t )
#endif

#ifndef __DBUS_CONEXUS__PARITY_ITERATOR_SUPPORT
  #define __DBUS_CONEXUS__PARITY_ITERATOR_SUPPORT
  DBUS_CXX_ITERATOR_SUPPORT( Conexus::Parity , int32_t )
#endif

#ifndef __DBUS_CONEXUS__PARITYERROR_ITERATOR_SUPPORT
  #define __DBUS_CONEXUS__PARITYERROR_ITERATOR_SUPPORT
  DBUS_CXX_ITERATOR_SUPPORT( Conexus::ParityError , int32_t )
#endif

#ifndef __DBUS_CONEXUS__READWRITE_ITERATOR_SUPPORT
  #define __DBUS_CONEXUS__READWRITE_ITERATOR_SUPPORT
  DBUS_CXX_ITERATOR_SUPPORT( Conexus::ReadWrite , int32_t )
#endif

#ifndef __DBUS_CONEXUS__SETOPTION_ITERATOR_SUPPORT
  #define __DBUS_CONEXUS__SETOPTION_ITERATOR_SUPPORT
  DBUS_CXX_ITERATOR_SUPPORT( Conexus::SetOption , int32_t )
#endif


namespace Conexus {
  namespace DBus {

    class TTYAdapter : public SerialAdapter
    {
      protected:
        TTYAdapter( Conexus::TTY* adaptee=NULL, const std::string& path="/Conexus/TTY"):
          SerialAdapter(adaptee, path),
          m_adaptee(adaptee)
        {

          ::DBus::MethodBase::pointer temp_method;

          temp_method = this->create_method<void>( "Conexus.TTY", "drain", sigc::mem_fun(*this, &TTYAdapter::drain_adapter_stub_v) );
          temp_method = this->create_method<void>( "Conexus.TTY", "flush_input", sigc::mem_fun(*this, &TTYAdapter::flush_input_adapter_stub_v) );
          temp_method = this->create_method<void>( "Conexus.TTY", "flush_output", sigc::mem_fun(*this, &TTYAdapter::flush_output_adapter_stub_v) );
          temp_method = this->create_method<void>( "Conexus.TTY", "flush", sigc::mem_fun(*this, &TTYAdapter::flush_adapter_stub_v) );
          temp_method = this->create_method<void>( "Conexus.TTY", "suspend_output", sigc::mem_fun(*this, &TTYAdapter::suspend_output_adapter_stub_v) );
          temp_method = this->create_method<void>( "Conexus.TTY", "restart_output", sigc::mem_fun(*this, &TTYAdapter::restart_output_adapter_stub_v) );
          temp_method = this->create_method<void>( "Conexus.TTY", "suspend_input", sigc::mem_fun(*this, &TTYAdapter::suspend_input_adapter_stub_v) );
          temp_method = this->create_method<void>( "Conexus.TTY", "restart_input", sigc::mem_fun(*this, &TTYAdapter::restart_input_adapter_stub_v) );
          temp_method = this->create_method<void,uint32_t,Conexus::SetOption>( "Conexus.TTY", "set_input_speed", sigc::mem_fun(*this, &TTYAdapter::set_input_speed_adapter_stub_vui) );
          temp_method->set_arg_name( 1, "speed" );
          temp_method->set_arg_name( 2, "set_option" );
          temp_method = this->create_method<void,uint32_t,Conexus::SetOption>( "Conexus.TTY", "set_output_speed", sigc::mem_fun(*this, &TTYAdapter::set_output_speed_adapter_stub_vui) );
          temp_method->set_arg_name( 1, "speed" );
          temp_method->set_arg_name( 2, "set_option" );
          temp_method = this->create_method<void,uint32_t,Conexus::SetOption>( "Conexus.TTY", "set_speed", sigc::mem_fun(*this, &TTYAdapter::set_speed_adapter_stub_vui) );
          temp_method->set_arg_name( 1, "speed" );
          temp_method->set_arg_name( 2, "set_option" );
          temp_method = this->create_method<uint32_t>( "Conexus.TTY", "input_speed", sigc::mem_fun(*this, &TTYAdapter::input_speed_adapter_stub_u) );
          temp_method->set_arg_name( 0, "speed" );
          temp_method = this->create_method<uint32_t>( "Conexus.TTY", "output_speed", sigc::mem_fun(*this, &TTYAdapter::output_speed_adapter_stub_u) );
          temp_method->set_arg_name( 0, "speed" );
          temp_method = this->create_method<Conexus::Parity>( "Conexus.TTY", "parity", sigc::mem_fun(*this, &TTYAdapter::parity_adapter_stub_i) );
          temp_method->set_arg_name( 0, "mode" );
          temp_method = this->create_method<void,Conexus::Parity>( "Conexus.TTY", "set_parity", sigc::mem_fun(*this, &TTYAdapter::set_parity_adapter_stub_vi) );
          temp_method->set_arg_name( 1, "mode" );
          temp_method = this->create_method<void,Conexus::Parity,Conexus::ParityError,Conexus::SetOption>( "Conexus.TTY", "set_parity", sigc::mem_fun(*this, &TTYAdapter::set_parity_adapter_stub_viii) );
          temp_method->set_arg_name( 1, "mode" );
          temp_method->set_arg_name( 2, "error_mode" );
          temp_method->set_arg_name( 3, "set_option" );
          temp_method = this->create_method<Conexus::ParityError>( "Conexus.TTY", "parity_error", sigc::mem_fun(*this, &TTYAdapter::parity_error_adapter_stub_i) );
          temp_method->set_arg_name( 0, "error" );
          temp_method = this->create_method<uint32_t>( "Conexus.TTY", "byte_size", sigc::mem_fun(*this, &TTYAdapter::byte_size_adapter_stub_u) );
          temp_method->set_arg_name( 0, "size" );
          temp_method = this->create_method<void,uint32_t,Conexus::SetOption>( "Conexus.TTY", "set_byte_size", sigc::mem_fun(*this, &TTYAdapter::set_byte_size_adapter_stub_vui) );
          temp_method->set_arg_name( 1, "size" );
          temp_method->set_arg_name( 2, "set_option" );
          temp_method = this->create_method<uint32_t>( "Conexus.TTY", "stop_bits", sigc::mem_fun(*this, &TTYAdapter::stop_bits_adapter_stub_u) );
          temp_method->set_arg_name( 0, "bits" );
          temp_method = this->create_method<void,uint32_t,Conexus::SetOption>( "Conexus.TTY", "set_stop_bits", sigc::mem_fun(*this, &TTYAdapter::set_stop_bits_adapter_stub_vui) );
          temp_method->set_arg_name( 1, "size" );
          temp_method->set_arg_name( 2, "set_option" );
          temp_method = this->create_method<Conexus::FlowControl>( "Conexus.TTY", "flow_control", sigc::mem_fun(*this, &TTYAdapter::flow_control_adapter_stub_i) );
          temp_method->set_arg_name( 0, "mode" );
          temp_method = this->create_method<void,Conexus::FlowControl,Conexus::SetOption>( "Conexus.TTY", "set_flow_control", sigc::mem_fun(*this, &TTYAdapter::set_flow_control_adapter_stub_vii) );
          temp_method->set_arg_name( 1, "mode" );
          temp_method->set_arg_name( 2, "set_option" );
          temp_method = this->create_method<bool>( "Conexus.TTY", "is_carrier_detect_enabled", sigc::mem_fun(*this, &TTYAdapter::is_carrier_detect_enabled_adapter_stub_b) );
          temp_method->set_arg_name( 0, "enabled" );
          temp_method = this->create_method<void,bool,Conexus::SetOption>( "Conexus.TTY", "enable_carrier_detect", sigc::mem_fun(*this, &TTYAdapter::enable_carrier_detect_adapter_stub_vbi) );
          temp_method->set_arg_name( 1, "enable" );
          temp_method->set_arg_name( 2, "set_option" );
          temp_method = this->create_method<bool>( "Conexus.TTY", "is_receiver_enabled", sigc::mem_fun(*this, &TTYAdapter::is_receiver_enabled_adapter_stub_b) );
          temp_method->set_arg_name( 0, "enabled" );
          temp_method = this->create_method<void,bool,Conexus::SetOption>( "Conexus.TTY", "enable_receiver", sigc::mem_fun(*this, &TTYAdapter::enable_receiver_adapter_stub_vbi) );
          temp_method->set_arg_name( 1, "enable" );
          temp_method->set_arg_name( 2, "set_option" );
          temp_method = this->create_method<void>( "Conexus.TTY", "open", sigc::mem_fun(*this, &TTYAdapter::open_adapter_stub_v) );
          temp_method = this->create_method<void,std::string,Conexus::ReadWrite>( "Conexus.TTY", "open", sigc::mem_fun(*this, &TTYAdapter::open_adapter_stub_vsi) );
          temp_method->set_arg_name( 1, "name" );
          temp_method->set_arg_name( 2, "mode" );
          temp_method = this->create_method<void,bool>( "Conexus.TTY", "close", sigc::mem_fun(*this, &TTYAdapter::close_adapter_stub_vb) );
          temp_method->set_arg_name( 1, "force" );
          temp_method = this->create_method<void,uint8_t>( "Conexus.TTY", "inject", sigc::mem_fun(*this, &TTYAdapter::inject_adapter_stub_vy) );
          temp_method->set_arg_name( 1, "data" );
          temp_method = this->create_method<void,std::string>( "Conexus.TTY", "inject", sigc::mem_fun(*this, &TTYAdapter::inject_adapter_stub_vs) );
          temp_method->set_arg_name( 1, "buffer" );
          temp_method = this->create_method<bool>( "Conexus.TTY", "is_reset_on_close_enabled", sigc::mem_fun(*this, &TTYAdapter::is_reset_on_close_enabled_adapter_stub_b) );
          temp_method->set_arg_name( 0, "enabled" );
          temp_method = this->create_method<void,bool>( "Conexus.TTY", "enable_reset_on_close", sigc::mem_fun(*this, &TTYAdapter::enable_reset_on_close_adapter_stub_vb) );
          temp_method->set_arg_name( 1, "reset" );
          m_signal_adapter_input_speed_changed = this->create_signal<void>("Conexus.TTY","input_speed_changed");
          if ( m_adaptee ) m_signal_adapter_connection_input_speed_changed = m_adaptee->signal_input_speed_changed().connect( *m_signal_adapter_input_speed_changed );
          m_signal_adapter_output_speed_changed = this->create_signal<void>("Conexus.TTY","output_speed_changed");
          if ( m_adaptee ) m_signal_adapter_connection_output_speed_changed = m_adaptee->signal_output_speed_changed().connect( *m_signal_adapter_output_speed_changed );
          m_signal_adapter_parity_changed = this->create_signal<void>("Conexus.TTY","parity_changed");
          if ( m_adaptee ) m_signal_adapter_connection_parity_changed = m_adaptee->signal_parity_changed().connect( *m_signal_adapter_parity_changed );
          m_signal_adapter_byte_size_changed = this->create_signal<void>("Conexus.TTY","byte_size_changed");
          if ( m_adaptee ) m_signal_adapter_connection_byte_size_changed = m_adaptee->signal_byte_size_changed().connect( *m_signal_adapter_byte_size_changed );
          m_signal_adapter_stop_bits_changed = this->create_signal<void>("Conexus.TTY","stop_bits_changed");
          if ( m_adaptee ) m_signal_adapter_connection_stop_bits_changed = m_adaptee->signal_stop_bits_changed().connect( *m_signal_adapter_stop_bits_changed );
          m_signal_adapter_flow_control_changed = this->create_signal<void>("Conexus.TTY","flow_control_changed");
          if ( m_adaptee ) m_signal_adapter_connection_flow_control_changed = m_adaptee->signal_flow_control_changed().connect( *m_signal_adapter_flow_control_changed );
          m_signal_adapter_carrier_detect_changed = this->create_signal<void>("Conexus.TTY","carrier_detect_changed");
          if ( m_adaptee ) m_signal_adapter_connection_carrier_detect_changed = m_adaptee->signal_carrier_detect_changed().connect( *m_signal_adapter_carrier_detect_changed );
          m_signal_adapter_receiver_changed = this->create_signal<void>("Conexus.TTY","receiver_changed");
          if ( m_adaptee ) m_signal_adapter_connection_receiver_changed = m_adaptee->signal_receiver_changed().connect( *m_signal_adapter_receiver_changed );
        }

      public:

        typedef DBusCxxPointer<TTYAdapter> pointer;

        static pointer create( Conexus::TTY* adaptee=NULL, const std::string& path="/Conexus/TTY")
          { return pointer( new TTYAdapter(adaptee, path)); }

        Conexus::TTY* adaptee() { return m_adaptee; }

        void set_adaptee( Conexus::TTY* adaptee ) {
          m_adaptee = adaptee;
          m_signal_adapter_connection_input_speed_changed.disconnect();
          m_signal_adapter_connection_output_speed_changed.disconnect();
          m_signal_adapter_connection_parity_changed.disconnect();
          m_signal_adapter_connection_byte_size_changed.disconnect();
          m_signal_adapter_connection_stop_bits_changed.disconnect();
          m_signal_adapter_connection_flow_control_changed.disconnect();
          m_signal_adapter_connection_carrier_detect_changed.disconnect();
          m_signal_adapter_connection_receiver_changed.disconnect();
          if ( m_adaptee ) m_signal_adapter_connection_input_speed_changed = m_adaptee->signal_input_speed_changed().connect( *m_signal_adapter_input_speed_changed );
          if ( m_adaptee ) m_signal_adapter_connection_output_speed_changed = m_adaptee->signal_output_speed_changed().connect( *m_signal_adapter_output_speed_changed );
          if ( m_adaptee ) m_signal_adapter_connection_parity_changed = m_adaptee->signal_parity_changed().connect( *m_signal_adapter_parity_changed );
          if ( m_adaptee ) m_signal_adapter_connection_byte_size_changed = m_adaptee->signal_byte_size_changed().connect( *m_signal_adapter_byte_size_changed );
          if ( m_adaptee ) m_signal_adapter_connection_stop_bits_changed = m_adaptee->signal_stop_bits_changed().connect( *m_signal_adapter_stop_bits_changed );
          if ( m_adaptee ) m_signal_adapter_connection_flow_control_changed = m_adaptee->signal_flow_control_changed().connect( *m_signal_adapter_flow_control_changed );
          if ( m_adaptee ) m_signal_adapter_connection_carrier_detect_changed = m_adaptee->signal_carrier_detect_changed().connect( *m_signal_adapter_carrier_detect_changed );
          if ( m_adaptee ) m_signal_adapter_connection_receiver_changed = m_adaptee->signal_receiver_changed().connect( *m_signal_adapter_receiver_changed );
        }

        void set_adaptee( Conexus::TTY& adaptee ) {
          this->set_adaptee(&adaptee);
        }


      private:

        Conexus::TTY* m_adaptee;


        void check_adaptee() { if ( not m_adaptee) throw ::DBus::ErrorInvalidAdaptee::create(); }

        void drain_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->drain(); }
        void flush_input_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->flush_input(); }
        void flush_output_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->flush_output(); }
        void flush_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->flush(); }
        void suspend_output_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->suspend_output(); }
        void restart_output_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->restart_output(); }
        void suspend_input_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->suspend_input(); }
        void restart_input_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->restart_input(); }
        void set_input_speed_adapter_stub_vui( uint32_t speed, Conexus::SetOption set_option ) { this->check_adaptee(); m_adaptee->set_input_speed( speed, set_option); }
        void set_output_speed_adapter_stub_vui( uint32_t speed, Conexus::SetOption set_option ) { this->check_adaptee(); m_adaptee->set_output_speed( speed, set_option); }
        void set_speed_adapter_stub_vui( uint32_t speed, Conexus::SetOption set_option ) { this->check_adaptee(); m_adaptee->set_speed( speed, set_option); }
        uint32_t input_speed_adapter_stub_u( ) { this->check_adaptee(); return m_adaptee->input_speed(); }
        uint32_t output_speed_adapter_stub_u( ) { this->check_adaptee(); return m_adaptee->output_speed(); }
        Conexus::Parity parity_adapter_stub_i( ) { this->check_adaptee(); return m_adaptee->parity(); }
        void set_parity_adapter_stub_vi( Conexus::Parity mode ) { this->check_adaptee(); m_adaptee->set_parity( mode); }
        void set_parity_adapter_stub_viii( Conexus::Parity mode, Conexus::ParityError error_mode, Conexus::SetOption set_option ) { this->check_adaptee(); m_adaptee->set_parity( mode, error_mode, set_option); }
        Conexus::ParityError parity_error_adapter_stub_i( ) { this->check_adaptee(); return m_adaptee->parity_error(); }
        uint32_t byte_size_adapter_stub_u( ) { this->check_adaptee(); return m_adaptee->byte_size(); }
        void set_byte_size_adapter_stub_vui( uint32_t size, Conexus::SetOption set_option ) { this->check_adaptee(); m_adaptee->set_byte_size( size, set_option); }
        uint32_t stop_bits_adapter_stub_u( ) { this->check_adaptee(); return m_adaptee->stop_bits(); }
        void set_stop_bits_adapter_stub_vui( uint32_t size, Conexus::SetOption set_option ) { this->check_adaptee(); m_adaptee->set_stop_bits( size, set_option); }
        Conexus::FlowControl flow_control_adapter_stub_i( ) { this->check_adaptee(); return m_adaptee->flow_control(); }
        void set_flow_control_adapter_stub_vii( Conexus::FlowControl mode, Conexus::SetOption set_option ) { this->check_adaptee(); m_adaptee->set_flow_control( mode, set_option); }
        bool is_carrier_detect_enabled_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->is_carrier_detect_enabled(); }
        void enable_carrier_detect_adapter_stub_vbi( bool enable, Conexus::SetOption set_option ) { this->check_adaptee(); m_adaptee->enable_carrier_detect( enable, set_option); }
        bool is_receiver_enabled_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->is_receiver_enabled(); }
        void enable_receiver_adapter_stub_vbi( bool enable, Conexus::SetOption set_option ) { this->check_adaptee(); m_adaptee->enable_receiver( enable, set_option); }
        void open_adapter_stub_v( ) { this->check_adaptee(); m_adaptee->open(); }
        void open_adapter_stub_vsi( std::string name, Conexus::ReadWrite mode ) { this->check_adaptee(); m_adaptee->open( name, mode); }
        void close_adapter_stub_vb( bool force ) { this->check_adaptee(); m_adaptee->close( force); }
        void inject_adapter_stub_vy( uint8_t data ) { this->check_adaptee(); m_adaptee->inject( data); }
        void inject_adapter_stub_vs( std::string buffer ) { this->check_adaptee(); m_adaptee->inject( buffer); }
        bool is_reset_on_close_enabled_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->is_reset_on_close_enabled(); }
        void enable_reset_on_close_adapter_stub_vb( bool reset ) { this->check_adaptee(); m_adaptee->enable_reset_on_close( reset); }
        ::DBus::signal<void>::pointer m_signal_adapter_input_speed_changed;
        sigc::connection m_signal_adapter_connection_input_speed_changed;
        ::DBus::signal<void>::pointer m_signal_adapter_output_speed_changed;
        sigc::connection m_signal_adapter_connection_output_speed_changed;
        ::DBus::signal<void>::pointer m_signal_adapter_parity_changed;
        sigc::connection m_signal_adapter_connection_parity_changed;
        ::DBus::signal<void>::pointer m_signal_adapter_byte_size_changed;
        sigc::connection m_signal_adapter_connection_byte_size_changed;
        ::DBus::signal<void>::pointer m_signal_adapter_stop_bits_changed;
        sigc::connection m_signal_adapter_connection_stop_bits_changed;
        ::DBus::signal<void>::pointer m_signal_adapter_flow_control_changed;
        sigc::connection m_signal_adapter_connection_flow_control_changed;
        ::DBus::signal<void>::pointer m_signal_adapter_carrier_detect_changed;
        sigc::connection m_signal_adapter_connection_carrier_detect_changed;
        ::DBus::signal<void>::pointer m_signal_adapter_receiver_changed;
        sigc::connection m_signal_adapter_connection_receiver_changed;


    };

  }
}

#endif
